# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.timestamp import Timestamp
from ...objects.types.actor import Actor
from ...objects.types.api import Api
from ...objects.types.attack import Attack
from ...objects.types.authorization import Authorization
from ...objects.types.cloud import Cloud
from ...objects.types.container import Container
from ...objects.types.database import Database
from ...objects.types.databucket import Databucket
from ...objects.types.device import Device
from ...objects.types.enrichment import Enrichment
from ...objects.types.firewall_rule import FirewallRule
from ...objects.types.idp import Idp
from ...objects.types.malware import Malware
from ...objects.types.metadata import Metadata
from ...objects.types.object import Object
from ...objects.types.observable import Observable
from ...objects.types.osint import Osint
from ...objects.types.policy import Policy
from ...objects.types.resource_details import ResourceDetails
from ...objects.types.table import Table
from .action_id import ActionId
from .activity_id import ActivityId
from .category_uid import CategoryUid
from .class_uid import ClassUid
from .confidence_id import ConfidenceId
from .disposition_id import DispositionId
from .risk_level_id import RiskLevelId
from .severity_id import SeverityId
from .status_id import StatusId
from .type_uid import TypeUid

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class CloudResourcesInventoryInfo(pydantic.BaseModel):
    """
    Cloud Resources Inventory Info events report cloud asset inventory data that is either logged or proactively collected. For example, use this event class when creating an inventory of cloud resource information from a Configuration Management Database (CMDB), Cyber Asset Attack Surface Management (CAASM), direct public cloud service provider APIs, Software-as-a-Service (SaaS) APIs, or otherwise.
    """

    action: typing.Optional[str] = pydantic.Field(default=None)
    """
    The normalized caption of <code>action_id</code>.
    """

    action_id: typing.Optional[ActionId] = pydantic.Field(default=None)
    """
    The action taken by a control or other policy-based system leading to an outcome or disposition. An unknown action may still correspond to a known disposition. Refer to <code>disposition_id</code> for the outcome of the action.
    """

    activity_id: ActivityId = pydantic.Field()
    """
    The normalized identifier of the activity that triggered the event.
    """

    activity_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event activity name, as defined by the activity_id.
    """

    actor: typing.Optional[Actor] = pydantic.Field(default=None)
    """
    The actor object describes details about the user/role/process that was the source of the activity. Note that this is not the threat actor of a campaign but may be part of a campaign.
    """

    api: typing.Optional[Api] = pydantic.Field(default=None)
    """
    Describes details about a typical API (Application Programming Interface) call.
    """

    attacks: typing.Optional[typing.List[Attack]] = pydantic.Field(default=None)
    """
    An array of <a target='_blank' href='https://attack.mitre.org'>MITRE ATT&CK®</a> objects describing identified tactics, techniques & sub-techniques.
    """

    authorizations: typing.Optional[typing.List[Authorization]] = pydantic.Field(default=None)
    """
    Provides details about an authorization, such as authorization outcome, and any associated policies related to the activity/event.
    """

    category_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event category name, as defined by category_uid value: <code>Discovery</code>.
    """

    category_uid: CategoryUid = pydantic.Field()
    """
    The category unique identifier of the event.
    """

    class_uid: ClassUid = pydantic.Field()
    """
    The unique identifier of a class. A class describes the attributes available in an event.
    """

    cloud: typing.Optional[Cloud] = pydantic.Field(default=None)
    """
    Cloud service provider or SaaS platform metadata about the cloud resource(s) that are being discovered by an inventory process.
    """

    confidence: typing.Optional[str] = pydantic.Field(default=None)
    """
    The confidence, normalized to the caption of the confidence_id value. In the case of 'Other', it is defined by the event source.
    """

    confidence_id: typing.Optional[ConfidenceId] = pydantic.Field(default=None)
    """
    The normalized confidence refers to the accuracy of the rule that created the finding. A rule with a low confidence means that the finding scope is wide and may create finding reports that may not be malicious in nature.
    """

    confidence_score: typing.Optional[int] = pydantic.Field(default=None)
    """
    The confidence score as reported by the event source.
    """

    container: typing.Optional[Container] = pydantic.Field(default=None)
    """
    A cloud-based container image or running container discovered by an inventory process.
    """

    count: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of times that events in the same logical group occurred during the event <strong>Start Time</strong> to <strong>End Time</strong> period.
    """

    custom_fields: typing.Optional[Object] = pydantic.Field(default=None)
    """
    A list of custom fields
    """

    database: typing.Optional[Database] = pydantic.Field(default=None)
    """
    A cloud-based database discovered by an inventory process.
    """

    databucket: typing.Optional[Databucket] = pydantic.Field(default=None)
    """
    A cloud-based data bucket or other object storage discovered by an inventory process.
    """

    device: typing.Optional[Device] = pydantic.Field(default=None)
    """
    An addressable device, computer system or host.
    """

    disposition: typing.Optional[str] = pydantic.Field(default=None)
    """
    The disposition name, normalized to the caption of the disposition_id value. In the case of 'Other', it is defined by the event source.
    """

    disposition_id: typing.Optional[DispositionId] = pydantic.Field(default=None)
    """
    Describes the outcome or action taken by a security control, such as access control checks, malware detections or various types of policy violations.
    """

    duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    The event duration or aggregate time, the amount of time the event covers from <code>start_time</code> to <code>end_time</code> in milliseconds.
    """

    end_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The end time of a time period, or the time of the most recent event included in the aggregate event.
    """

    end_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The end time of a time period, or the time of the most recent event included in the aggregate event.
    """

    enrichments: typing.Optional[typing.List[Enrichment]] = pydantic.Field(default=None)
    """
    The additional information from an external data source, which is associated with the event or a finding. For example add location information for the IP address in the DNS answers:</p><code>[{"name": "answers.ip", "value": "92.24.47.250", "type": "location", "data": {"city": "Socotra", "continent": "Asia", "coordinates": [-25.4153, 17.0743], "country": "YE", "desc": "Yemen"}}]</code>
    """

    firewall_rule: typing.Optional[FirewallRule] = pydantic.Field(default=None)
    """
    The firewall rule that pertains to the control that triggered the event, if applicable.
    """

    idp: typing.Optional[Idp] = pydantic.Field(default=None)
    """
    The Identity Provider that is being discovered by an inventory process, or that is related to the cloud resource(s) being discovered by an inventory process.
    """

    is_alert: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates that the event is considered to be an alertable signal. Should be set to <code>true</code> if <code>disposition_id = Alert</code> among other dispositions, and/or <code>risk_level_id</code> or <code>severity_id</code> of the event is elevated. Not all control events will be alertable, for example if <code>disposition_id = Exonerated</code> or <code>disposition_id = Allowed</code>.
    """

    malware: typing.Optional[typing.List[Malware]] = pydantic.Field(default=None)
    """
    A list of Malware objects, describing details about the identified malware.
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the event/finding, as defined by the source.
    """

    metadata: Metadata = pydantic.Field()
    """
    The metadata associated with the event or a finding.
    """

    observables: typing.Optional[typing.List[Observable]] = pydantic.Field(default=None)
    """
    The observables associated with the event or a finding.
    """

    osint: typing.Optional[typing.List[Osint]] = pydantic.Field(default=None)
    """
    The OSINT (Open Source Intelligence) object contains details related to an indicator such as the indicator itself, related indicators, geolocation, registrar information, subdomains, analyst commentary, and other contextual information. This information can be used to further enrich a detection or finding by providing decisioning support to other analysts and engineers.
    """

    policy: typing.Optional[Policy] = pydantic.Field(default=None)
    """
    The policy that pertains to the control that triggered the event, if applicable. For example the name of an anti-malware policy or an access control policy.
    """

    raw_data: typing.Optional[str] = pydantic.Field(default=None)
    """
    The raw event/finding data as received from the source.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    The cloud region where the resource is located, e.g., <code>us-isof-south-1</code>, <code>eastus2</code>, <code>us-central1</code>, etc.
    """

    resources: typing.Optional[typing.List[ResourceDetails]] = pydantic.Field(default=None)
    """
    The cloud resource(s) that are being discovered by an inventory process. Use this object if there is not a direct object match in the class.
    """

    risk_details: typing.Optional[str] = pydantic.Field(default=None)
    """
    Describes the risk associated with the finding.
    """

    risk_level: typing.Optional[str] = pydantic.Field(default=None)
    """
    The risk level, normalized to the caption of the risk_level_id value.
    """

    risk_level_id: typing.Optional[RiskLevelId] = pydantic.Field(default=None)
    """
    The normalized risk level id.
    """

    risk_score: typing.Optional[int] = pydantic.Field(default=None)
    """
    The risk score as reported by the event source.
    """

    severity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event/finding severity, normalized to the caption of the severity_id value. In the case of 'Other', it is defined by the source.
    """

    severity_id: SeverityId = pydantic.Field()
    """
    <p>The normalized identifier of the event/finding severity.</p>The normalized severity is a measurement the effort and expense required to manage and resolve an event or incident. Smaller numerical values represent lower impact events, and larger numerical values represent higher impact events.
    """

    start_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The start time of a time period, or the time of the least recent event included in the aggregate event.
    """

    start_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The start time of a time period, or the time of the least recent event included in the aggregate event.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event status, normalized to the caption of the status_id value. In the case of 'Other', it is defined by the event source.
    """

    status_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event status code, as reported by the event source.<br /><br />For example, in a Windows Failed Authentication event, this would be the value of 'Failure Code', e.g. 0x18.
    """

    status_detail: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status detail contains additional information about the event/finding outcome.
    """

    status_id: typing.Optional[StatusId] = pydantic.Field(default=None)
    """
    The normalized identifier of the event status.
    """

    table: typing.Optional[Table] = pydantic.Field(default=None)
    """
    A cloud-based database table discovered by an inventory process.
    """

    time: Timestamp = pydantic.Field()
    """
    The normalized event occurrence time or the finding creation time.
    """

    time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The normalized event occurrence time or the finding creation time.
    """

    timezone_offset: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of minutes that the reported event <code>time</code> is ahead or behind UTC, in the range -1,080 to +1,080.
    """

    type_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event/finding type name, as defined by the type_uid.
    """

    type_uid: TypeUid = pydantic.Field()
    """
    The event/finding type ID. It identifies the event's semantics and structure. The value is calculated by the logging system as: <code>class_uid \* 100 + activity_id</code>.
    """

    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the event schema. The names and values of those attributes are specific to the event source.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...engine.types.api_response import ApiResponse
from ...ticketing.types.priority import Priority

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class TicketingWebhookResponse(ApiResponse):
    ticket_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the ticket in the ticketing system.
    """

    entity_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    Date pertiment to webhook operation e.g. if its TicketCreated would be creation date. This comes directly from the ticketing system without any date formatting changes.
    """

    priority: typing.Optional[Priority] = pydantic.Field(default=None)
    """
    The priority of the ticket.
    """

    short_description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Short Description of the ticket in the ticketing system.
    """

    raw_payload: str = pydantic.Field()
    """
    The Raw JSON that was sent by the ticketing system for this webhook event.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ConnectionState(str, enum.Enum):
    CONNECT = "Connect"
    DISCONNECT = "Disconnect"

    def visit(self, connect: typing.Callable[[], T_Result], disconnect: typing.Callable[[], T_Result]) -> T_Result:
        if self is ConnectionState.CONNECT:
            return connect()
        if self is ConnectionState.DISCONNECT:
            return disconnect()



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...credentials.types.token_credential import TokenCredential
from ...credentials.types.token_credential_id import TokenCredentialId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Rapid7InsightCloudCredential_Token(TokenCredential):
    type: typing.Literal["token"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class Rapid7InsightCloudCredential_TokenId(pydantic.BaseModel):
    type: typing.Literal["token_id"]
    value: TokenCredentialId

    class Config:
        frozen = True
        smart_union = True


Rapid7InsightCloudCredential = typing.Union[Rapid7InsightCloudCredential_Token, Rapid7InsightCloudCredential_TokenId]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Image(pydantic.BaseModel):
    """
    The Image object provides a description of a specific Virtual Machine (VM) or Container image. Defined by D3FEND <a target='_blank' href='https://d3fend.mitre.org/dao/artifact/d3f:ContainerImage/'>d3f:ContainerImage</a>.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The image labels.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The image name. For example: <code>elixir</code>.
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    The full path to the image file.
    """

    tag: typing.Optional[str] = pydantic.Field(default=None)
    """
    The image tag. For example: <code>1.11-alpine</code>.
    """

    uid: str = pydantic.Field()
    """
    The unique image ID. For example: <code>77af4d6b9913</code>.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...account_base.types.account_id import AccountId
from ...accounts.types.account import Account
from ...integration_base.types.integration_id import IntegrationId
from ...integrations.types.integration import Integration

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Status(pydantic.BaseModel):
    """
    Status object
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Time object was originally created
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    Last time object was updated
    """

    account_id: AccountId = pydantic.Field()
    """
    Account associated with this status. Use the expand=accounts parameter with the List API to expand the Account to the full object
    """

    account: typing.Optional[Account] = pydantic.Field(default=None)
    """
    When using the expand option on the List API, the full account object is included in the response
    """

    integration_id: IntegrationId = pydantic.Field()
    """
    Integration associated with this status. Use the expand=integrations parameter with the List API to expand the Integration to the full object
    """

    integration: typing.Optional[Integration] = pydantic.Field(default=None)
    """
    When using the expand option on the List API, the full integration object is included in the response
    """

    status: str = pydantic.Field()
    """
    The current status of the notification.
    """

    requests: int = pydantic.Field()
    """
    Request count
    """

    failed: int = pydantic.Field()
    """
    Failed count
    """

    cpu_time: int = pydantic.Field()
    """
    Cpu time in microseconds
    """

    db_ops: int = pydantic.Field()
    """
    Database operations count
    """

    api_ops: int = pydantic.Field()
    """
    API operations count
    """

    in_bytes: int = pydantic.Field()
    """
    API input byte count
    """

    out_bytes: int = pydantic.Field()
    """
    API output byte count
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.hostname import Hostname
from ...base.types.port import Port
from ...base.types.url_string import UrlString
from .url_category_ids import UrlCategoryIds

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Url(pydantic.BaseModel):
    """
    The Uniform Resource Locator (URL) object describes the characteristics of a URL.
    """

    categories: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The Website categorization names, as defined by <code>category_ids</code> enum values.
    """

    category_ids: typing.Optional[typing.List[UrlCategoryIds]] = pydantic.Field(default=None)
    """
    The Website categorization identifiers.
    """

    domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The domain portion of the URL. For example: <code>example.com</code> in <code>https://sub.example.com</code>.
    """

    hostname: typing.Optional[Hostname] = pydantic.Field(default=None)
    """
    The URL host as extracted from the URL. For example: <code>www.example.com</code> from <code>www.example.com/download/trouble</code>.
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    The URL path as extracted from the URL. For example: <code>/download/trouble</code> from <code>www.example.com/download/trouble</code>.
    """

    port: typing.Optional[Port] = pydantic.Field(default=None)
    """
    The URL port. For example: <code>80</code>.
    """

    query_string: typing.Optional[str] = pydantic.Field(default=None)
    """
    The query portion of the URL. For example: the query portion of the URL <code>http://www.example.com/search?q=bad&sort=date</code> is <code>q=bad&sort=date</code>.
    """

    resource_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The context in which a resource was retrieved in a web request.
    """

    scheme: typing.Optional[str] = pydantic.Field(default=None)
    """
    The scheme portion of the URL. For example: <code>http</code>, <code>https</code>, <code>ftp</code>, or <code>sftp</code>.
    """

    subdomain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The subdomain portion of the URL. For example: <code>sub</code> in <code>https://sub.example.com</code> or <code>sub2.sub1</code> in <code>https://sub2.sub1.example.com</code>.
    """

    url_string: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The URL string. See RFC 1738. For example: <code>http://www.example.com/download/trouble.exe</code>. Note: The URL path should not populate the URL string.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .api_has_status import ApiHasStatus
from .api_query_response import ApiQueryResponse
from .api_response import ApiResponse
from .base_resource_request import BaseResourceRequest
from .meta_api import MetaApi
from .meta_api_primary_response import MetaApiPrimaryResponse
from .meta_api_response import MetaApiResponse
from .meta_response import MetaResponse
from .meta_stats import MetaStats
from .object import Object
from .option_value import OptionValue
from .order_options import OrderOptions
from .query_status import QueryStatus

__all__ = [
    "ApiHasStatus",
    "ApiQueryResponse",
    "ApiResponse",
    "BaseResourceRequest",
    "MetaApi",
    "MetaApiPrimaryResponse",
    "MetaApiResponse",
    "MetaResponse",
    "MetaStats",
    "Object",
    "OptionValue",
    "OrderOptions",
    "QueryStatus",
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CreateIntegrationTokenRequest,
    CreateIntegrationTokenResponse,
    CreateSynqlyIntegrationsTokenRequest,
    CreateSynqlyIntegrationsTokenResponse,
    CreateTokenRequest,
    CreateTokenResponse,
    GetTokenResponse,
    ListTokensResponse,
    RefreshToken,
    RefreshTokenResponse,
    ResetTokenResponse,
    SynqlyIntegrationsTokenResponse,
)

__all__ = [
    "CreateIntegrationTokenRequest",
    "CreateIntegrationTokenResponse",
    "CreateSynqlyIntegrationsTokenRequest",
    "CreateSynqlyIntegrationsTokenResponse",
    "CreateTokenRequest",
    "CreateTokenResponse",
    "GetTokenResponse",
    "ListTokensResponse",
    "RefreshToken",
    "RefreshTokenResponse",
    "ResetTokenResponse",
    "SynqlyIntegrationsTokenResponse",
]



# This file was auto-generated by Fern from our API Definition.

from .activity_id import ActivityId
from .category_uid import CategoryUid
from .class_uid import ClassUid
from .confidence_id import ConfidenceId
from .impact_id import ImpactId
from .incident_finding import IncidentFinding
from .priority_id import PriorityId
from .severity_id import SeverityId
from .status_id import StatusId
from .type_uid import TypeUid
from .verdict_id import VerdictId

__all__ = [
    "ActivityId",
    "CategoryUid",
    "ClassUid",
    "ConfidenceId",
    "ImpactId",
    "IncidentFinding",
    "PriorityId",
    "SeverityId",
    "StatusId",
    "TypeUid",
    "VerdictId",
]



# This file was auto-generated by Fern from our API Definition.

from . import events
from .events import (
    AccountChange,
    ApiActivity,
    Authentication,
    CloudResourcesInventoryInfo,
    ComplianceFinding,
    DetectionFinding,
    DnsActivity,
    EntityManagement,
    FileActivity,
    GroupManagement,
    IncidentFinding,
    InventoryInfo,
    ModuleActivity,
    NetworkActivity,
    ProcessActivity,
    ScheduledJobActivity,
    SecurityFinding,
    VulnerabilityFinding,
    WebResourceAccessActivity,
)

__all__ = [
    "AccountChange",
    "ApiActivity",
    "Authentication",
    "CloudResourcesInventoryInfo",
    "ComplianceFinding",
    "DetectionFinding",
    "DnsActivity",
    "EntityManagement",
    "FileActivity",
    "GroupManagement",
    "IncidentFinding",
    "InventoryInfo",
    "ModuleActivity",
    "NetworkActivity",
    "ProcessActivity",
    "ScheduledJobActivity",
    "SecurityFinding",
    "VulnerabilityFinding",
    "WebResourceAccessActivity",
    "events",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ResourceType(str, enum.Enum):
    ACCOUNT = "account"
    BRIDGE = "bridge"
    CREDENTIAL = "credential"
    INTEGRATION_POINT = "integration_point"
    INTEGRATION = "integration"
    MEMBER = "member"
    OPERATION = "operation"
    ORGANIZATION_WEBHOOK = "organization_webhook"
    ROLE = "role"
    SUB_ORG = "sub_org"
    TOKEN = "token"
    TRANSFORM = "transform"

    def visit(
        self,
        account: typing.Callable[[], T_Result],
        bridge: typing.Callable[[], T_Result],
        credential: typing.Callable[[], T_Result],
        integration_point: typing.Callable[[], T_Result],
        integration: typing.Callable[[], T_Result],
        member: typing.Callable[[], T_Result],
        operation: typing.Callable[[], T_Result],
        organization_webhook: typing.Callable[[], T_Result],
        role: typing.Callable[[], T_Result],
        sub_org: typing.Callable[[], T_Result],
        token: typing.Callable[[], T_Result],
        transform: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ResourceType.ACCOUNT:
            return account()
        if self is ResourceType.BRIDGE:
            return bridge()
        if self is ResourceType.CREDENTIAL:
            return credential()
        if self is ResourceType.INTEGRATION_POINT:
            return integration_point()
        if self is ResourceType.INTEGRATION:
            return integration()
        if self is ResourceType.MEMBER:
            return member()
        if self is ResourceType.OPERATION:
            return operation()
        if self is ResourceType.ORGANIZATION_WEBHOOK:
            return organization_webhook()
        if self is ResourceType.ROLE:
            return role()
        if self is ResourceType.SUB_ORG:
            return sub_org()
        if self is ResourceType.TOKEN:
            return token()
        if self is ResourceType.TRANSFORM:
            return transform()



# This file was auto-generated by Fern from our API Definition.

from .audit import Audit
from .audit_type import AuditType
from .http_method import HttpMethod
from .list_audit_events_response import ListAuditEventsResponse

__all__ = ["Audit", "AuditType", "HttpMethod", "ListAuditEventsResponse"]



# This file was auto-generated by Fern from our API Definition.

from .....v_1_3_0.resources.vulnerabilityfinding.resources.classes.types.vulnerability_finding import (
    VulnerabilityFinding as resources_ocsf_resources_v_1_3_0_resources_vulnerabilityfinding_resources_classes_types_vulnerability_finding_VulnerabilityFinding,
)

VulnerabilityFinding = resources_ocsf_resources_v_1_3_0_resources_vulnerabilityfinding_resources_classes_types_vulnerability_finding_VulnerabilityFinding



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Permissions(str, enum.Enum):
    ADMINISTRATOR = "administrator"
    VIEWER = "viewer"
    MEMBER = "member"
    ACCOUNT_MANAGER = "account-manager"
    CONNECT_UI = "connect-ui"
    TOKEN_ISSUER = "token-issuer"
    MCP_INTEGRATIONS_USE_ONLY = "mcp-integrations-use-only"
    """
    Permission set that provides the minimum level of access necessary for MCP use. Gives read access to accounts and integrations and the ability to use any Connector API.
    """

    def visit(
        self,
        administrator: typing.Callable[[], T_Result],
        viewer: typing.Callable[[], T_Result],
        member: typing.Callable[[], T_Result],
        account_manager: typing.Callable[[], T_Result],
        connect_ui: typing.Callable[[], T_Result],
        token_issuer: typing.Callable[[], T_Result],
        mcp_integrations_use_only: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is Permissions.ADMINISTRATOR:
            return administrator()
        if self is Permissions.VIEWER:
            return viewer()
        if self is Permissions.MEMBER:
            return member()
        if self is Permissions.ACCOUNT_MANAGER:
            return account_manager()
        if self is Permissions.CONNECT_UI:
            return connect_ui()
        if self is Permissions.TOKEN_ISSUER:
            return token_issuer()
        if self is Permissions.MCP_INTEGRATIONS_USE_ONLY:
            return mcp_integrations_use_only()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class OperationStatus(str, enum.Enum):
    SCHEDULED = "SCHEDULED"
    PROCESSING = "PROCESSING"
    CANCELLED = "CANCELLED"
    COMPLETE = "COMPLETE"

    def visit(
        self,
        scheduled: typing.Callable[[], T_Result],
        processing: typing.Callable[[], T_Result],
        cancelled: typing.Callable[[], T_Result],
        complete: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is OperationStatus.SCHEDULED:
            return scheduled()
        if self is OperationStatus.PROCESSING:
            return processing()
        if self is OperationStatus.CANCELLED:
            return cancelled()
        if self is OperationStatus.COMPLETE:
            return complete()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.file_name import FileName
from ...base.types.timestamp import Timestamp
from .digital_signature import DigitalSignature
from .file_confidentiality_id import FileConfidentialityId
from .file_type_id import FileTypeId
from .fingerprint import Fingerprint
from .object import Object
from .product import Product
from .user import User

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class File(pydantic.BaseModel):
    """
    The File object represents the metadata associated with a file stored in a computer system. It encompasses information about the file itself, including its attributes, properties, and organizational details. Defined by D3FEND <a target='_blank' href='https://next.d3fend.mitre.org/dao/artifact/d3f:File/'>d3f:File</a>.
    """

    accessed_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the file was last accessed.
    """

    accessed_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the file was last accessed.
    """

    accessor: typing.Optional[User] = pydantic.Field(default=None)
    """
    The name of the user who last accessed the object.
    """

    attributes: typing.Optional[int] = pydantic.Field(default=None)
    """
    The bitmask value that represents the file attributes.
    """

    company_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the company that published the file. For example: <code>Microsoft Corporation</code>.
    """

    confidentiality: typing.Optional[str] = pydantic.Field(default=None)
    """
    The file content confidentiality, normalized to the confidentiality_id value. In the case of 'Other', it is defined by the event source.
    """

    confidentiality_id: typing.Optional[FileConfidentialityId] = pydantic.Field(default=None)
    """
    The normalized identifier of the file content confidentiality indicator.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the file was created.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the file was created.
    """

    creator: typing.Optional[User] = pydantic.Field(default=None)
    """
    The user that created the file.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the file, as returned by file system. For example: the description as returned by the Unix file command or the Windows file type.
    """

    ext: typing.Optional[str] = pydantic.Field(default=None)
    """
    The extension of the file, excluding the leading dot. For example: <code>exe</code> from <code>svchost.exe</code>, or <code>gz</code> from <code>export.tar.gz</code>.
    """

    hashes: typing.Optional[typing.List[Fingerprint]] = pydantic.Field(default=None)
    """
    An array of hash attributes.
    """

    is_system: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The indication of whether the object is part of the operating system.
    """

    mime_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Multipurpose Internet Mail Extensions (MIME) type of the file, if applicable.
    """

    modified_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the file was last modified.
    """

    modified_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the file was last modified.
    """

    modifier: typing.Optional[User] = pydantic.Field(default=None)
    """
    The user that last modified the file.
    """

    name: FileName = pydantic.Field()
    """
    The name of the file. For example: <code>svchost.exe</code>
    """

    owner: typing.Optional[User] = pydantic.Field(default=None)
    """
    The user that owns the file/object.
    """

    parent_folder: typing.Optional[str] = pydantic.Field(default=None)
    """
    The parent folder in which the file resides. For example: <code>c:\windows\system32</code>
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    The full path to the file. For example: <code>c:\windows\system32\svchost.exe</code>.
    """

    product: typing.Optional[Product] = pydantic.Field(default=None)
    """
    The product that created or installed the file.
    """

    security_descriptor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The object security descriptor.
    """

    signature: typing.Optional[DigitalSignature] = pydantic.Field(default=None)
    """
    The digital signature of the file.
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size of data, in bytes.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The file type.
    """

    type_id: FileTypeId = pydantic.Field()
    """
    The file type ID.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the file as defined by the storage system, such the file system file ID.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The file version. For example: <code>8.0.7601.17514</code>.
    """

    xattributes: typing.Optional[Object] = pydantic.Field(default=None)
    """
    An unordered collection of zero or more name/value pairs where each pair represents a file or folder extended attribute.</p>For example: Windows alternate data stream attributes (ADS stream name, ADS size, etc.), user-defined or application-defined attributes, ACL, owner, primary group, etc. Examples from DCS: </p><ul><li><strong>ads_name</strong></li><li><strong>ads_size</strong></li><li><strong>dacl</strong></li><li><strong>owner</strong></li><li><strong>primary_group</strong></li><li><strong>link_name</strong> - name of the link associated to the file.</li><li><strong>hard_link_count</strong> - the number of links that are associated to the file.</li></ul>
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from ...ocsf.resources.v_1_3_0.resources.configstate.resources.classes.types.config_state import ConfigState

PostureScore = ConfigState



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.process_name import ProcessName
from ...base.types.timestamp import Timestamp
from .container import Container
from .file import File
from .group import Group
from .object import Object
from .process_integrity_id import ProcessIntegrityId
from .session import Session
from .user import User

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Process(pydantic.BaseModel):
    """
    The Process object describes a running instance of a launched program. Defined by D3FEND <a target='_blank' href='https://d3fend.mitre.org/dao/artifact/d3f:Process/'>d3f:Process</a>.
    """

    auid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The audit user assigned at login by the audit subsystem.
    """

    cmd_line: typing.Optional[str] = pydantic.Field(default=None)
    """
    The full command line used to launch an application, service, process, or job. For example: <code>ssh user@10.0.0.10</code>. If the command line is unavailable or missing, the empty string <code>''</code> is to be used.
    """

    container: typing.Optional[Container] = pydantic.Field(default=None)
    """
    The information describing an instance of a container. A container is a prepackaged, portable system image that runs isolated on an existing system using a container runtime like containerd.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the process was created/started.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the process was created/started.
    """

    egid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The effective group under which this process is running.
    """

    euid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The effective user under which this process is running.
    """

    file: typing.Optional[File] = pydantic.Field(default=None)
    """
    The process file object.
    """

    group: typing.Optional[Group] = pydantic.Field(default=None)
    """
    The group under which this process is running.
    """

    integrity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The process integrity level, normalized to the caption of the integrity_id value. In the case of 'Other', it is defined by the event source (Windows only).
    """

    integrity_id: typing.Optional[ProcessIntegrityId] = pydantic.Field(default=None)
    """
    The normalized identifier of the process integrity level (Windows only).
    """

    lineage: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The lineage of the process, represented by a list of paths for each ancestor process. For example: <code>['/usr/sbin/sshd', '/usr/bin/bash', '/usr/bin/whoami']</code>.
    """

    loaded_modules: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of loaded module names.
    """

    name: typing.Optional[ProcessName] = pydantic.Field(default=None)
    """
    The friendly name of the process, for example: <code>Notepad++</code>.
    """

    namespace_pid: typing.Optional[int] = pydantic.Field(default=None)
    """
    If running under a process namespace (such as in a container), the process identifier within that process namespace.
    """

    parent_process: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The parent process of this process object. It is recommended to only populate this field for the first process object, to prevent deep nesting.
    """

    pid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The process identifier, as reported by the operating system. Process ID (PID) is a number used by the operating system to uniquely identify an active process.
    """

    sandbox: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the containment jail (i.e., sandbox). For example, hardened_ps, high_security_ps, oracle_ps, netsvcs_ps, or default_ps.
    """

    session: typing.Optional[Session] = pydantic.Field(default=None)
    """
    The user session under which this process is running.
    """

    terminated_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the process was terminated.
    """

    terminated_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the process was terminated.
    """

    tid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The Identifier of the thread associated with the event, as returned by the operating system.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier for this process assigned by the producer (tool). Facilitates correlation of a process event with other events for that process.
    """

    user: typing.Optional[User] = pydantic.Field(default=None)
    """
    The user under which this process is running.
    """

    xattributes: typing.Optional[Object] = pydantic.Field(default=None)
    """
    An unordered collection of zero or more name/value pairs that represent a process extended attribute.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .identity_class_ov import IdentityClassOv
from .implementation_language_ov import ImplementationLanguageOv
from .indicator_type_ov import IndicatorTypeOv
from .malware_capabilities_ov import MalwareCapabilitiesOv
from .malware_type_ov import MalwareTypeOv
from .pattern_type_ov import PatternTypeOv
from .processor_architecture_ov import ProcessorArchitectureOv
from .statement import Statement
from .tlp import Tlp

__all__ = [
    "IdentityClassOv",
    "ImplementationLanguageOv",
    "IndicatorTypeOv",
    "MalwareCapabilitiesOv",
    "MalwareTypeOv",
    "PatternTypeOv",
    "ProcessorArchitectureOv",
    "Statement",
    "Tlp",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .file import File
from .fingerprint import Fingerprint
from .long_string import LongString
from .script_type_id import ScriptTypeId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Script(pydantic.BaseModel):
    """
    The Script object describes a script or command that can be executed by a shell, script engine, or interpreter. Examples include Bash, JavsScript, PowerShell, Python, VBScript, etc. Note that the term <em>script</em> here denotes not only a script contained within a file but also a script or command typed interactively by a user, supplied on the command line, or provided by some other file-less mechanism.
    """

    file: typing.Optional[File] = pydantic.Field(default=None)
    """
    Present if this script is associated with a file. Not present in the case of a file-less script.
    """

    hashes: typing.Optional[typing.List[Fingerprint]] = pydantic.Field(default=None)
    """
    An array of the script's cryptographic hashes. Note that these hashes are calculated on the script in its original encoding, and not on the normalized UTF-8 encoding found in the <code>script_content</code> attribute.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the script or macro, independent of the containing file, used for tracking, auditing, and security analysis.
    """

    parent_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    This attribute relates a sub-script to a parent script having the matching <code>uid</code> attribute. In the case of PowerShell, sub-script execution can be identified by matching the activity correlation ID of the raw ETW events provided by the OS.
    """

    script_content: LongString = pydantic.Field()
    """
    The script content, normalized to UTF-8 encoding irrespective of its original encoding. When emitting this attribute, it may be appropriate to truncate large scripts. When consuming this attribute, large scripts should be anticipated.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The script type, normalized to the caption of the <code>type_id</code> value. In the case of 'Other', it is defined by the event source.
    """

    type_id: ScriptTypeId = pydantic.Field()
    """
    The normalized script type ID.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Some script engines assign a unique ID to each individual execution of a given script. This attribute captures that unique ID. In the case of PowerShell, the unique ID corresponds to the <code>ScriptBlockId</code> in the raw ETW events provided by the OS.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .assets_armis_centrix import AssetsArmisCentrix
from .assets_armis_centrix_mock import AssetsArmisCentrixMock
from .assets_axonius import AssetsAxonius
from .assets_crowd_strike import AssetsCrowdStrike
from .assets_nozomi_vantage import AssetsNozomiVantage
from .assets_nozomi_vantage_mock import AssetsNozomiVantageMock
from .assets_qualys_cloud import AssetsQualysCloud
from .assets_qualys_cloud_mock import AssetsQualysCloudMock
from .assets_service_now import AssetsServiceNow
from .assets_service_now_mock import AssetsServiceNowMock
from .assets_sevco import AssetsSevco
from .assets_tanium_cloud import AssetsTaniumCloud
from .assets_tanium_cloud_mock import AssetsTaniumCloudMock
from .cloud_security_crowd_strike import CloudSecurityCrowdStrike
from .cloud_security_defender import CloudSecurityDefender
from .edr_crowd_strike import EdrCrowdStrike
from .edr_defender import EdrDefender
from .edr_malwarebytes import EdrMalwarebytes
from .edr_sentinel_one import EdrSentinelOne
from .edr_sophos import EdrSophos
from .edr_tanium import EdrTanium
from .identity_entra_id import IdentityEntraId
from .identity_google import IdentityGoogle
from .identity_okta import IdentityOkta
from .identity_ping_one import IdentityPingOne
from .notifications_jira import NotificationsJira
from .notifications_mock import NotificationsMock
from .notifications_slack import NotificationsSlack
from .notifications_slack_webhook import NotificationsSlackWebhook
from .notifications_teams import NotificationsTeams
from .siem_crowdstrike import SiemCrowdstrike
from .siem_elasticsearch import SiemElasticsearch
from .siem_google_chronicle import SiemGoogleChronicle
from .siem_google_security_operations import SiemGoogleSecurityOperations
from .siem_mock import SiemMock
from .siem_open_search import SiemOpenSearch
from .siem_q_radar import SiemQRadar
from .siem_rapid_7_insight_idr import SiemRapid7InsightIdr
from .siem_sentinel import SiemSentinel
from .siem_splunk import SiemSplunk
from .siem_sumo_logic import SiemSumoLogic
from .sink_aws_security_lake import SinkAwsSecurityLake
from .sink_aws_sqs import SinkAwsSqs
from .sink_azure_monitor_logs import SinkAzureMonitorLogs
from .sink_crowdstrike_hec import SinkCrowdstrikeHec
from .sink_elasticsearch import SinkElasticsearch
from .sink_google_sec_ops import SinkGoogleSecOps
from .sink_google_security_operations import SinkGoogleSecurityOperations
from .sink_mock import SinkMock
from .sink_open_search import SinkOpenSearch
from .sink_splunk import SinkSplunk
from .storage_aws_s_3 import StorageAwsS3
from .storage_azure_blob import StorageAzureBlob
from .storage_gcs import StorageGcs
from .storage_mock import StorageMock
from .ticketing_autotask import TicketingAutotask
from .ticketing_freshdesk import TicketingFreshdesk
from .ticketing_jira import TicketingJira
from .ticketing_mock import TicketingMock
from .ticketing_pager_duty import TicketingPagerDuty
from .ticketing_service_now import TicketingServiceNow
from .ticketing_service_now_sir import TicketingServiceNowSir
from .ticketing_torq import TicketingTorq
from .ticketing_zendesk import TicketingZendesk
from .vulnerabilities_crowd_strike import VulnerabilitiesCrowdStrike
from .vulnerabilities_nucleus import VulnerabilitiesNucleus
from .vulnerabilities_qualys_cloud import VulnerabilitiesQualysCloud
from .vulnerabilities_qualys_cloud_mock import VulnerabilitiesQualysCloudMock
from .vulnerabilities_rapid_7_insight_cloud import VulnerabilitiesRapid7InsightCloud
from .vulnerabilities_rapid_7_insight_cloud_mock import VulnerabilitiesRapid7InsightCloudMock
from .vulnerabilities_tanium_cloud import VulnerabilitiesTaniumCloud
from .vulnerabilities_tanium_cloud_mock import VulnerabilitiesTaniumCloudMock
from .vulnerabilities_tenable_cloud import VulnerabilitiesTenableCloud


class ProviderConfig_AssetsArmisCentrix(AssetsArmisCentrix):
    type: typing.Literal["assets_armis_centrix"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_AssetsArmisCentrixMock(AssetsArmisCentrixMock):
    type: typing.Literal["assets_armis_centrix_mock"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_AssetsAxonius(AssetsAxonius):
    type: typing.Literal["assets_axonius"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_AssetsCrowdstrike(AssetsCrowdStrike):
    type: typing.Literal["assets_crowdstrike"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_AssetsNozomiVantage(AssetsNozomiVantage):
    type: typing.Literal["assets_nozomi_vantage"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_AssetsNozomiVantageMock(AssetsNozomiVantageMock):
    type: typing.Literal["assets_nozomi_vantage_mock"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_AssetsQualysCloud(AssetsQualysCloud):
    type: typing.Literal["assets_qualys_cloud"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_AssetsQualysCloudMock(AssetsQualysCloudMock):
    type: typing.Literal["assets_qualys_cloud_mock"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_AssetsServicenow(AssetsServiceNow):
    type: typing.Literal["assets_servicenow"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_AssetsServicenowMock(AssetsServiceNowMock):
    type: typing.Literal["assets_servicenow_mock"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_AssetsSevco(AssetsSevco):
    type: typing.Literal["assets_sevco"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_AssetsTaniumCloud(AssetsTaniumCloud):
    type: typing.Literal["assets_tanium_cloud"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_AssetsTaniumCloudMock(AssetsTaniumCloudMock):
    type: typing.Literal["assets_tanium_cloud_mock"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_CloudsecurityCrowdstrike(CloudSecurityCrowdStrike):
    type: typing.Literal["cloudsecurity_crowdstrike"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_CloudsecurityDefender(CloudSecurityDefender):
    type: typing.Literal["cloudsecurity_defender"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_EdrCrowdstrike(EdrCrowdStrike):
    type: typing.Literal["edr_crowdstrike"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_EdrDefender(EdrDefender):
    type: typing.Literal["edr_defender"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_EdrMalwarebytes(EdrMalwarebytes):
    type: typing.Literal["edr_malwarebytes"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_EdrSentinelone(EdrSentinelOne):
    type: typing.Literal["edr_sentinelone"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_EdrSophos(EdrSophos):
    type: typing.Literal["edr_sophos"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_EdrTanium(EdrTanium):
    type: typing.Literal["edr_tanium"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_IdentityEntraId(IdentityEntraId):
    type: typing.Literal["identity_entra_id"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_IdentityGoogle(IdentityGoogle):
    type: typing.Literal["identity_google"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_IdentityOkta(IdentityOkta):
    type: typing.Literal["identity_okta"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_IdentityPingone(IdentityPingOne):
    type: typing.Literal["identity_pingone"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_NotificationsJira(NotificationsJira):
    type: typing.Literal["notifications_jira"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_NotificationsMockNotifications(NotificationsMock):
    type: typing.Literal["notifications_mock_notifications"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_NotificationsSlack(NotificationsSlack):
    type: typing.Literal["notifications_slack"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_NotificationsSlackWebhook(NotificationsSlackWebhook):
    type: typing.Literal["notifications_slack_webhook"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_NotificationsTeams(NotificationsTeams):
    type: typing.Literal["notifications_teams"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SiemCrowdstrike(SiemCrowdstrike):
    type: typing.Literal["siem_crowdstrike"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SiemElasticsearch(SiemElasticsearch):
    type: typing.Literal["siem_elasticsearch"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SiemGoogleChronicle(SiemGoogleChronicle):
    type: typing.Literal["siem_google_chronicle"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SiemGoogleSecurityOperations(SiemGoogleSecurityOperations):
    type: typing.Literal["siem_google_security_operations"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SiemMockSiem(SiemMock):
    type: typing.Literal["siem_mock_siem"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SiemOpensearch(SiemOpenSearch):
    type: typing.Literal["siem_opensearch"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SiemQRadar(SiemQRadar):
    type: typing.Literal["siem_q_radar"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SiemRapid7Insightidr(SiemRapid7InsightIdr):
    type: typing.Literal["siem_rapid7_insightidr"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SiemSentinel(SiemSentinel):
    type: typing.Literal["siem_sentinel"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SiemSplunk(SiemSplunk):
    type: typing.Literal["siem_splunk"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SiemSumoLogic(SiemSumoLogic):
    type: typing.Literal["siem_sumo_logic"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SinkAwsSecurityLake(SinkAwsSecurityLake):
    type: typing.Literal["sink_aws_security_lake"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SinkAwsSqs(SinkAwsSqs):
    type: typing.Literal["sink_aws_sqs"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SinkAzureMonitorLogs(SinkAzureMonitorLogs):
    type: typing.Literal["sink_azure_monitor_logs"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SinkCrowdstrikeHec(SinkCrowdstrikeHec):
    type: typing.Literal["sink_crowdstrike_hec"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SinkElasticsearch(SinkElasticsearch):
    type: typing.Literal["sink_elasticsearch"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SinkGoogleSecOps(SinkGoogleSecOps):
    type: typing.Literal["sink_google_sec_ops"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SinkGoogleSecurityOperations(SinkGoogleSecurityOperations):
    type: typing.Literal["sink_google_security_operations"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SinkMockSink(SinkMock):
    type: typing.Literal["sink_mock_sink"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SinkOpensearch(SinkOpenSearch):
    type: typing.Literal["sink_opensearch"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_SinkSplunk(SinkSplunk):
    type: typing.Literal["sink_splunk"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_StorageAwsS3(StorageAwsS3):
    type: typing.Literal["storage_aws_s3"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_StorageAzureBlob(StorageAzureBlob):
    type: typing.Literal["storage_azure_blob"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_StorageGcs(StorageGcs):
    type: typing.Literal["storage_gcs"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_StorageMockStorage(StorageMock):
    type: typing.Literal["storage_mock_storage"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_TicketingAutotask(TicketingAutotask):
    type: typing.Literal["ticketing_autotask"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_TicketingFreshdesk(TicketingFreshdesk):
    type: typing.Literal["ticketing_freshdesk"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_TicketingJira(TicketingJira):
    type: typing.Literal["ticketing_jira"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_TicketingMockTicketing(TicketingMock):
    type: typing.Literal["ticketing_mock_ticketing"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_TicketingPagerduty(TicketingPagerDuty):
    type: typing.Literal["ticketing_pagerduty"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_TicketingServicenow(TicketingServiceNow):
    type: typing.Literal["ticketing_servicenow"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_TicketingServicenowSir(TicketingServiceNowSir):
    type: typing.Literal["ticketing_servicenow_sir"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_TicketingTorq(TicketingTorq):
    type: typing.Literal["ticketing_torq"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_TicketingZendesk(TicketingZendesk):
    type: typing.Literal["ticketing_zendesk"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_VulnerabilitiesCrowdstrike(VulnerabilitiesCrowdStrike):
    type: typing.Literal["vulnerabilities_crowdstrike"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_VulnerabilitiesNucleus(VulnerabilitiesNucleus):
    type: typing.Literal["vulnerabilities_nucleus"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_VulnerabilitiesQualysCloud(VulnerabilitiesQualysCloud):
    type: typing.Literal["vulnerabilities_qualys_cloud"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_VulnerabilitiesQualysCloudMock(VulnerabilitiesQualysCloudMock):
    type: typing.Literal["vulnerabilities_qualys_cloud_mock"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_VulnerabilitiesRapid7InsightCloud(VulnerabilitiesRapid7InsightCloud):
    type: typing.Literal["vulnerabilities_rapid7_insight_cloud"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_VulnerabilitiesRapid7InsightCloudMock(VulnerabilitiesRapid7InsightCloudMock):
    type: typing.Literal["vulnerabilities_rapid7_insight_cloud_mock"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_VulnerabilitiesTaniumCloud(VulnerabilitiesTaniumCloud):
    type: typing.Literal["vulnerabilities_tanium_cloud"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_VulnerabilitiesTaniumCloudMock(VulnerabilitiesTaniumCloudMock):
    type: typing.Literal["vulnerabilities_tanium_cloud_mock"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ProviderConfig_VulnerabilitiesTenableCloud(VulnerabilitiesTenableCloud):
    type: typing.Literal["vulnerabilities_tenable_cloud"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


ProviderConfig = typing.Union[
    ProviderConfig_AssetsArmisCentrix,
    ProviderConfig_AssetsArmisCentrixMock,
    ProviderConfig_AssetsAxonius,
    ProviderConfig_AssetsCrowdstrike,
    ProviderConfig_AssetsNozomiVantage,
    ProviderConfig_AssetsNozomiVantageMock,
    ProviderConfig_AssetsQualysCloud,
    ProviderConfig_AssetsQualysCloudMock,
    ProviderConfig_AssetsServicenow,
    ProviderConfig_AssetsServicenowMock,
    ProviderConfig_AssetsSevco,
    ProviderConfig_AssetsTaniumCloud,
    ProviderConfig_AssetsTaniumCloudMock,
    ProviderConfig_CloudsecurityCrowdstrike,
    ProviderConfig_CloudsecurityDefender,
    ProviderConfig_EdrCrowdstrike,
    ProviderConfig_EdrDefender,
    ProviderConfig_EdrMalwarebytes,
    ProviderConfig_EdrSentinelone,
    ProviderConfig_EdrSophos,
    ProviderConfig_EdrTanium,
    ProviderConfig_IdentityEntraId,
    ProviderConfig_IdentityGoogle,
    ProviderConfig_IdentityOkta,
    ProviderConfig_IdentityPingone,
    ProviderConfig_NotificationsJira,
    ProviderConfig_NotificationsMockNotifications,
    ProviderConfig_NotificationsSlack,
    ProviderConfig_NotificationsSlackWebhook,
    ProviderConfig_NotificationsTeams,
    ProviderConfig_SiemCrowdstrike,
    ProviderConfig_SiemElasticsearch,
    ProviderConfig_SiemGoogleChronicle,
    ProviderConfig_SiemGoogleSecurityOperations,
    ProviderConfig_SiemMockSiem,
    ProviderConfig_SiemOpensearch,
    ProviderConfig_SiemQRadar,
    ProviderConfig_SiemRapid7Insightidr,
    ProviderConfig_SiemSentinel,
    ProviderConfig_SiemSplunk,
    ProviderConfig_SiemSumoLogic,
    ProviderConfig_SinkAwsSecurityLake,
    ProviderConfig_SinkAwsSqs,
    ProviderConfig_SinkAzureMonitorLogs,
    ProviderConfig_SinkCrowdstrikeHec,
    ProviderConfig_SinkElasticsearch,
    ProviderConfig_SinkGoogleSecOps,
    ProviderConfig_SinkGoogleSecurityOperations,
    ProviderConfig_SinkMockSink,
    ProviderConfig_SinkOpensearch,
    ProviderConfig_SinkSplunk,
    ProviderConfig_StorageAwsS3,
    ProviderConfig_StorageAzureBlob,
    ProviderConfig_StorageGcs,
    ProviderConfig_StorageMockStorage,
    ProviderConfig_TicketingAutotask,
    ProviderConfig_TicketingFreshdesk,
    ProviderConfig_TicketingJira,
    ProviderConfig_TicketingMockTicketing,
    ProviderConfig_TicketingPagerduty,
    ProviderConfig_TicketingServicenow,
    ProviderConfig_TicketingServicenowSir,
    ProviderConfig_TicketingTorq,
    ProviderConfig_TicketingZendesk,
    ProviderConfig_VulnerabilitiesCrowdstrike,
    ProviderConfig_VulnerabilitiesNucleus,
    ProviderConfig_VulnerabilitiesQualysCloud,
    ProviderConfig_VulnerabilitiesQualysCloudMock,
    ProviderConfig_VulnerabilitiesRapid7InsightCloud,
    ProviderConfig_VulnerabilitiesRapid7InsightCloudMock,
    ProviderConfig_VulnerabilitiesTaniumCloud,
    ProviderConfig_VulnerabilitiesTaniumCloudMock,
    ProviderConfig_VulnerabilitiesTenableCloud,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class BridgeSelector_Id(pydantic.BaseModel):
    type: typing.Literal["id"]
    value: str

    class Config:
        frozen = True
        smart_union = True


class BridgeSelector_Labels(pydantic.BaseModel):
    type: typing.Literal["labels"]
    value: typing.List[str]

    class Config:
        frozen = True
        smart_union = True


BridgeSelector = typing.Union[BridgeSelector_Id, BridgeSelector_Labels]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class CategoryId(str, enum.Enum):
    """
    Id of the Integrations category
    """

    ASSETS = "assets"
    CLOUDSECURITY = "cloudsecurity"
    EDR = "edr"
    IDENTITY = "identity"
    NOTIFICATIONS = "notifications"
    SIEM = "siem"
    SINK = "sink"
    STORAGE = "storage"
    TICKETING = "ticketing"
    VULNERABILITIES = "vulnerabilities"

    def visit(
        self,
        assets: typing.Callable[[], T_Result],
        cloudsecurity: typing.Callable[[], T_Result],
        edr: typing.Callable[[], T_Result],
        identity: typing.Callable[[], T_Result],
        notifications: typing.Callable[[], T_Result],
        siem: typing.Callable[[], T_Result],
        sink: typing.Callable[[], T_Result],
        storage: typing.Callable[[], T_Result],
        ticketing: typing.Callable[[], T_Result],
        vulnerabilities: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is CategoryId.ASSETS:
            return assets()
        if self is CategoryId.CLOUDSECURITY:
            return cloudsecurity()
        if self is CategoryId.EDR:
            return edr()
        if self is CategoryId.IDENTITY:
            return identity()
        if self is CategoryId.NOTIFICATIONS:
            return notifications()
        if self is CategoryId.SIEM:
            return siem()
        if self is CategoryId.SINK:
            return sink()
        if self is CategoryId.STORAGE:
            return storage()
        if self is CategoryId.TICKETING:
            return ticketing()
        if self is CategoryId.VULNERABILITIES:
            return vulnerabilities()



# This file was auto-generated by Fern from our API Definition.

from .create_integration_token_request import CreateIntegrationTokenRequest
from .create_integration_token_response import CreateIntegrationTokenResponse
from .create_synqly_integrations_token_request import CreateSynqlyIntegrationsTokenRequest
from .create_synqly_integrations_token_response import CreateSynqlyIntegrationsTokenResponse
from .create_token_request import CreateTokenRequest
from .create_token_response import CreateTokenResponse
from .get_token_response import GetTokenResponse
from .list_tokens_response import ListTokensResponse
from .refresh_token import RefreshToken
from .refresh_token_response import RefreshTokenResponse
from .reset_token_response import ResetTokenResponse
from .synqly_integrations_token_response import SynqlyIntegrationsTokenResponse

__all__ = [
    "CreateIntegrationTokenRequest",
    "CreateIntegrationTokenResponse",
    "CreateSynqlyIntegrationsTokenRequest",
    "CreateSynqlyIntegrationsTokenResponse",
    "CreateTokenRequest",
    "CreateTokenResponse",
    "GetTokenResponse",
    "ListTokensResponse",
    "RefreshToken",
    "RefreshTokenResponse",
    "ResetTokenResponse",
    "SynqlyIntegrationsTokenResponse",
]



# This file was auto-generated by Fern from our API Definition.

from .query_cloud_resource_inventory_response import QueryCloudResourceInventoryResponse
from .query_compliance_findings_response import QueryComplianceFindingsResponse
from .query_events_response import QueryEventsResponse
from .query_ioms_response import QueryIomsResponse

__all__ = [
    "QueryCloudResourceInventoryResponse",
    "QueryComplianceFindingsResponse",
    "QueryEventsResponse",
    "QueryIomsResponse",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class EmailAuth(pydantic.BaseModel):
    """
    The Email Authentication object describes the Sender Policy Framework (SPF), DomainKeys Identified Mail (DKIM) and Domain-based Message Authentication, Reporting and Conformance (DMARC) attributes of an email.
    """

    dkim: typing.Optional[str] = pydantic.Field(default=None)
    """
    The DomainKeys Identified Mail (DKIM) status of the email.
    """

    dkim_domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The DomainKeys Identified Mail (DKIM) signing domain of the email.
    """

    dkim_signature: typing.Optional[str] = pydantic.Field(default=None)
    """
    The DomainKeys Identified Mail (DKIM) signature used by the sending/receiving system.
    """

    dmarc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Domain-based Message Authentication, Reporting and Conformance (DMARC) status of the email.
    """

    dmarc_override: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Domain-based Message Authentication, Reporting and Conformance (DMARC) override action.
    """

    dmarc_policy: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Domain-based Message Authentication, Reporting and Conformance (DMARC) policy status.
    """

    spf: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Sender Policy Framework (SPF) status of the email.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ImplementationLanguageOv(str, enum.Enum):
    APPLESCRIPT = "applescript"
    BASH = "bash"
    C = "c"
    CPP = "cpp"
    CSHARP = "csharp"
    GO = "go"
    JAVA = "java"
    JAVASCRIPT = "javascript"
    LUA = "lua"
    OBJECTIVE_C = "objective_c"
    PERL = "perl"
    PHP = "php"
    POWERSHELL = "powershell"
    PYTHON = "python"
    RUBY = "ruby"
    SCALA = "scala"
    SWIFT = "swift"
    TYPESCRIPT = "typescript"
    VISUAL_BASIC = "visual_basic"
    X_86_32 = "x86_32"
    X_86_64 = "x86_64"

    def visit(
        self,
        applescript: typing.Callable[[], T_Result],
        bash: typing.Callable[[], T_Result],
        c: typing.Callable[[], T_Result],
        cpp: typing.Callable[[], T_Result],
        csharp: typing.Callable[[], T_Result],
        go: typing.Callable[[], T_Result],
        java: typing.Callable[[], T_Result],
        javascript: typing.Callable[[], T_Result],
        lua: typing.Callable[[], T_Result],
        objective_c: typing.Callable[[], T_Result],
        perl: typing.Callable[[], T_Result],
        php: typing.Callable[[], T_Result],
        powershell: typing.Callable[[], T_Result],
        python: typing.Callable[[], T_Result],
        ruby: typing.Callable[[], T_Result],
        scala: typing.Callable[[], T_Result],
        swift: typing.Callable[[], T_Result],
        typescript: typing.Callable[[], T_Result],
        visual_basic: typing.Callable[[], T_Result],
        x_86_32: typing.Callable[[], T_Result],
        x_86_64: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ImplementationLanguageOv.APPLESCRIPT:
            return applescript()
        if self is ImplementationLanguageOv.BASH:
            return bash()
        if self is ImplementationLanguageOv.C:
            return c()
        if self is ImplementationLanguageOv.CPP:
            return cpp()
        if self is ImplementationLanguageOv.CSHARP:
            return csharp()
        if self is ImplementationLanguageOv.GO:
            return go()
        if self is ImplementationLanguageOv.JAVA:
            return java()
        if self is ImplementationLanguageOv.JAVASCRIPT:
            return javascript()
        if self is ImplementationLanguageOv.LUA:
            return lua()
        if self is ImplementationLanguageOv.OBJECTIVE_C:
            return objective_c()
        if self is ImplementationLanguageOv.PERL:
            return perl()
        if self is ImplementationLanguageOv.PHP:
            return php()
        if self is ImplementationLanguageOv.POWERSHELL:
            return powershell()
        if self is ImplementationLanguageOv.PYTHON:
            return python()
        if self is ImplementationLanguageOv.RUBY:
            return ruby()
        if self is ImplementationLanguageOv.SCALA:
            return scala()
        if self is ImplementationLanguageOv.SWIFT:
            return swift()
        if self is ImplementationLanguageOv.TYPESCRIPT:
            return typescript()
        if self is ImplementationLanguageOv.VISUAL_BASIC:
            return visual_basic()
        if self is ImplementationLanguageOv.X_86_32:
            return x_86_32()
        if self is ImplementationLanguageOv.X_86_64:
            return x_86_64()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...common.types.id import Id
from ...role_base.types.adhoc_role import AdhocRole
from ...role_base.types.role_name import RoleName

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Permission(pydantic.BaseModel):
    role_binding: typing.Optional[typing.List[RoleName]] = pydantic.Field(default=None)
    """
    Roles granted to this token.
    """

    adhoc_role: typing.Optional[AdhocRole] = pydantic.Field(default=None)
    """
    Adhoc role granted to this token.
    """

    resource_id: Id = pydantic.Field()
    """
    ID of the resource that this permission grants access to.
    """

    resource_type: str = pydantic.Field()
    """
    Type of the resource that this permission grants access to. Must be one of the following: "organization, "integration"
    """

    parent_id: Id = pydantic.Field()
    """
    Token parentId
    """

    id: Id = pydantic.Field()
    """
    Token Id
    """

    organization_id: Id = pydantic.Field()
    """
    Token organizationId
    """

    root_organization_id: typing.Optional[Id] = pydantic.Field(default=None)
    """
    Token root organizationId
    """

    member_id: Id = pydantic.Field()
    """
    Token memberId
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .....v_1_3_0.resources.networkactivity.resources.classes.types.network_activity import (
    NetworkActivity as resources_ocsf_resources_v_1_3_0_resources_networkactivity_resources_classes_types_network_activity_NetworkActivity,
)

NetworkActivity = (
    resources_ocsf_resources_v_1_3_0_resources_networkactivity_resources_classes_types_network_activity_NetworkActivity
)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ProviderAvailability(str, enum.Enum):
    IN_DEVELOPMENT = "in-development"
    """
    Currently in active development.
    """

    PRE_RELEASE = "pre-release"
    """
    Available for use, but not ready for production.
    """

    DEPRECATED = "deprecated"
    """
    Will be removed in the future.
    """

    GENERALLY_AVAILABLE = "generally-available"
    """
    Stable and available for production use.
    """

    def visit(
        self,
        in_development: typing.Callable[[], T_Result],
        pre_release: typing.Callable[[], T_Result],
        deprecated: typing.Callable[[], T_Result],
        generally_available: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ProviderAvailability.IN_DEVELOPMENT:
            return in_development()
        if self is ProviderAvailability.PRE_RELEASE:
            return pre_release()
        if self is ProviderAvailability.DEPRECATED:
            return deprecated()
        if self is ProviderAvailability.GENERALLY_AVAILABLE:
            return generally_available()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.url_string import UrlString

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class SubTechnique(pydantic.BaseModel):
    """
    The MITRE ATT&CK® Sub Technique object describes the sub technique ID and/or name associated to an attack, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK® Matrix</a>.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the attack sub technique, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK® Matrix</a>. For example: <code>Scanning IP Blocks</code>.
    """

    src_url: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The versioned permalink of the attack sub technique, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK® Matrix</a>. For example: <code>https://attack.mitre.org/versions/v14/techniques/T1595/001/</code>.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the attack sub technique, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK® Matrix</a>. For example: <code>T1595.001</code>.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .container import Container

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Request(pydantic.BaseModel):
    """
    The Request Elements object describes characteristics of an API request.
    """

    containers: typing.Optional[typing.List[Container]] = pydantic.Field(default=None)
    """
    When working with containerized applications, the set of containers which write to the standard the output of a particular logging driver. For example, this may be the set of containers involved in handling api requests and responses for a containerized application.
    """

    data: typing.Optional[typing.Any] = pydantic.Field(default=None)
    """
    The additional data that is associated with the api request.
    """

    flags: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The communication flags that are associated with the api request.
    """

    uid: str = pydantic.Field()
    """
    The unique request identifier.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ..account_base.types.account_id import AccountId
from ..common.errors.bad_request_error import BadRequestError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.internal_server_error import InternalServerError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.types.problem import Problem
from ..integration_base.types.integration_id import IntegrationId
from .types.get_integration_timeseries import GetIntegrationTimeseries
from .types.get_status_response import GetStatusResponse
from .types.get_status_timeseries import GetStatusTimeseries
from .types.list_status_events_response import ListStatusEventsResponse
from .types.list_status_options import ListStatusOptions
from .types.list_status_response import ListStatusResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class StatusClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        expand: typing.Optional[typing.Union[ListStatusOptions, typing.Sequence[ListStatusOptions]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListStatusResponse:
        """
        Returns all matching `Status` objects.

        Parameters:
            - limit: typing.Optional[int]. Number of `Status` objects to return in this page. Defaults to 100.

            - start_after: typing.Optional[str]. Return `Status` objects starting after this `account_id,integration_id`.

            - order: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Select a field to order the results by. Defaults to `account_id,integration_id`. To control the direction of the sorting, append
                                                                               `[asc]` or `[desc]` to the field name. For example, `name[desc]` will sort the results by `name` in descending order.
                                                                               The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the
                                                                               ordering is applied in the order the fields are specified.
            - filter: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter.
                                                                                If used more than once, the queries are ANDed together.
            - expand: typing.Optional[typing.Union[ListStatusOptions, typing.Sequence[ListStatusOptions]]]. Expand the status result with the related integration and/or account information.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/status"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "start_after": start_after,
                        "order": order,
                        "filter": filter,
                        "expand": expand,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListStatusResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        account_id: AccountId,
        integration_id: IntegrationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetStatusResponse:
        """
        Returns the integration `Status` object.

        Parameters:
            - account_id: AccountId.

            - integration_id: IntegrationId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/status/{jsonable_encoder(account_id)}/{jsonable_encoder(integration_id)}",
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetStatusResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def reset(
        self,
        account_id: AccountId,
        integration_id: IntegrationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Resets the integration `Status` object.

        Parameters:
            - account_id: AccountId.

            - integration_id: IntegrationId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/status/{jsonable_encoder(account_id)}/{jsonable_encoder(integration_id)}",
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_events(
        self,
        account_id: AccountId,
        integration_id: IntegrationId,
        *,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        order: typing.Optional[str] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListStatusEventsResponse:
        """
        Returns integration `Status` object list of `StatusEvent` objects.

        Parameters:
            - account_id: AccountId.

            - integration_id: IntegrationId.

            - limit: typing.Optional[int]. Number of `StatusEvent` objects to return in this page. Defaults to 100.

            - start_after: typing.Optional[str]. Return `StatusEvent` objects starting after this `created_at`.

            - order: typing.Optional[str]. The order defaults to created_at[asc] and can changed to descending order by specifying created_at[desc].

            - filter: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter.
                                                                                If used more than once, the queries are ANDed together.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/status/{jsonable_encoder(account_id)}/{jsonable_encoder(integration_id)}/events",
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "start_after": start_after,
                        "order": order,
                        "filter": filter,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListStatusEventsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_timeseries(self, *, request_options: typing.Optional[RequestOptions] = None) -> GetStatusTimeseries:
        """
        Returns organization last hour usage timeseries.

        Parameters:
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/status/timeseries"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetStatusTimeseries, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_integration_timeseries(
        self,
        account_id: AccountId,
        integration_id: IntegrationId,
        *,
        interval: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetIntegrationTimeseries:
        """
        Returns organization last hour usage timeseries.

        Parameters:
            - account_id: AccountId.

            - integration_id: IntegrationId.

            - interval: typing.Optional[str]. [hour] provide most recent 24 hour timeseries. default: hour

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/status/{jsonable_encoder(account_id)}/{jsonable_encoder(integration_id)}/timeseries",
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "interval": interval,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetIntegrationTimeseries, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncStatusClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        expand: typing.Optional[typing.Union[ListStatusOptions, typing.Sequence[ListStatusOptions]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListStatusResponse:
        """
        Returns all matching `Status` objects.

        Parameters:
            - limit: typing.Optional[int]. Number of `Status` objects to return in this page. Defaults to 100.

            - start_after: typing.Optional[str]. Return `Status` objects starting after this `account_id,integration_id`.

            - order: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Select a field to order the results by. Defaults to `account_id,integration_id`. To control the direction of the sorting, append
                                                                               `[asc]` or `[desc]` to the field name. For example, `name[desc]` will sort the results by `name` in descending order.
                                                                               The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the
                                                                               ordering is applied in the order the fields are specified.
            - filter: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter.
                                                                                If used more than once, the queries are ANDed together.
            - expand: typing.Optional[typing.Union[ListStatusOptions, typing.Sequence[ListStatusOptions]]]. Expand the status result with the related integration and/or account information.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/status"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "start_after": start_after,
                        "order": order,
                        "filter": filter,
                        "expand": expand,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListStatusResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        account_id: AccountId,
        integration_id: IntegrationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetStatusResponse:
        """
        Returns the integration `Status` object.

        Parameters:
            - account_id: AccountId.

            - integration_id: IntegrationId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/status/{jsonable_encoder(account_id)}/{jsonable_encoder(integration_id)}",
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetStatusResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def reset(
        self,
        account_id: AccountId,
        integration_id: IntegrationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Resets the integration `Status` object.

        Parameters:
            - account_id: AccountId.

            - integration_id: IntegrationId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/status/{jsonable_encoder(account_id)}/{jsonable_encoder(integration_id)}",
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_events(
        self,
        account_id: AccountId,
        integration_id: IntegrationId,
        *,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        order: typing.Optional[str] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListStatusEventsResponse:
        """
        Returns integration `Status` object list of `StatusEvent` objects.

        Parameters:
            - account_id: AccountId.

            - integration_id: IntegrationId.

            - limit: typing.Optional[int]. Number of `StatusEvent` objects to return in this page. Defaults to 100.

            - start_after: typing.Optional[str]. Return `StatusEvent` objects starting after this `created_at`.

            - order: typing.Optional[str]. The order defaults to created_at[asc] and can changed to descending order by specifying created_at[desc].

            - filter: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter.
                                                                                If used more than once, the queries are ANDed together.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/status/{jsonable_encoder(account_id)}/{jsonable_encoder(integration_id)}/events",
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "start_after": start_after,
                        "order": order,
                        "filter": filter,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListStatusEventsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_timeseries(self, *, request_options: typing.Optional[RequestOptions] = None) -> GetStatusTimeseries:
        """
        Returns organization last hour usage timeseries.

        Parameters:
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/status/timeseries"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetStatusTimeseries, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_integration_timeseries(
        self,
        account_id: AccountId,
        integration_id: IntegrationId,
        *,
        interval: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetIntegrationTimeseries:
        """
        Returns organization last hour usage timeseries.

        Parameters:
            - account_id: AccountId.

            - integration_id: IntegrationId.

            - interval: typing.Optional[str]. [hour] provide most recent 24 hour timeseries. default: hour

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/status/{jsonable_encoder(account_id)}/{jsonable_encoder(integration_id)}/timeseries",
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "interval": interval,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetIntegrationTimeseries, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CreateDeviceRequest,
    CreateDeviceRequestGeneric,
    CreateDeviceResponse,
    CreateDeviceResponseGeneric,
    Device,
    GetLabelsResponse,
    GetLabelsResponseGeneric,
    Label,
    QueryDevicesResponse,
    QueryDevicesResponseGeneric,
)

__all__ = [
    "CreateDeviceRequest",
    "CreateDeviceRequestGeneric",
    "CreateDeviceResponse",
    "CreateDeviceResponseGeneric",
    "Device",
    "GetLabelsResponse",
    "GetLabelsResponseGeneric",
    "Label",
    "QueryDevicesResponse",
    "QueryDevicesResponseGeneric",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.email_address import EmailAddress
from ...base.types.ip_address import IpAddress
from .file import File
from .http_header import HttpHeader
from .url import Url

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Email(pydantic.BaseModel):
    """
    The Email object describes the email metadata such as sender, recipients, and direction, and can include embedded URLs and files.
    """

    cc: typing.Optional[typing.List[EmailAddress]] = pydantic.Field(default=None)
    """
    The machine-readable email header Cc values, as defined by RFC 5322. For example <code>example.user@usersdomain.com</code>.
    """

    cc_mailboxes: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The human-readable email header Cc Mailbox values. For example <code>'Example User &lt;example.user@usersdomain.com&gt;'</code>.
    """

    delivered_to: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The machine-readable <strong>Delivered-To</strong> email header field. For example <code>example.user@usersdomain.com</code>
    """

    delivered_to_list: typing.Optional[typing.List[EmailAddress]] = pydantic.Field(default=None)
    """
    The machine-readable <strong>Delivered-To</strong> email header values. For example <code>example.user@usersdomain.com</code>
    """

    files: typing.Optional[typing.List[File]] = pydantic.Field(default=None)
    """
    The files embedded or attached to the email.
    """

    from_: typing.Optional[EmailAddress] = pydantic.Field(alias="from", default=None)
    """
    The machine-readable email header From values, as defined by RFC 5322. For example <code>example.user@usersdomain.com</code>
    """

    from_mailbox: typing.Optional[str] = pydantic.Field(default=None)
    """
    The human-readable email header From Mailbox value. For example <code>'Example User &lt;example.user@usersdomain.com&gt;'</code>.
    """

    http_headers: typing.Optional[typing.List[HttpHeader]] = pydantic.Field(default=None)
    """
    Additional HTTP headers of an HTTP request or response.
    """

    message_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email header Message-ID value, as defined by RFC 5322.
    """

    raw_header: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email authentication header.
    """

    reply_to: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The machine-readable email header Reply-To values, as defined by RFC 5322. For example <code>example.user@usersdomain.com</code>
    """

    reply_to_mailboxes: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The human-readable email header Reply To Mailbox values. For example <code>'Example User &lt;example.user@usersdomain.com&gt;'</code>.
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size in bytes of the email, including attachments.
    """

    smtp_from: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The value of the SMTP MAIL FROM command.
    """

    smtp_to: typing.Optional[typing.List[EmailAddress]] = pydantic.Field(default=None)
    """
    The value of the SMTP envelope RCPT TO command.
    """

    subject: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email header Subject value, as defined by RFC 5322.
    """

    to: typing.Optional[typing.List[EmailAddress]] = pydantic.Field(default=None)
    """
    The machine-readable email header To values, as defined by RFC 5322. For example <code>example.user@usersdomain.com</code>
    """

    to_mailboxes: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The human-readable email header To Mailbox values. For example <code>'Example User &lt;example.user@usersdomain.com&gt;'</code>.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the email thread.
    """

    urls: typing.Optional[typing.List[Url]] = pydantic.Field(default=None)
    """
    The URLs embedded in the email.
    """

    x_originating_ip: typing.Optional[typing.List[IpAddress]] = pydantic.Field(default=None)
    """
    The X-Originating-IP header identifying the emails originating IP address(es).
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...account_base.types.account_id import AccountId
from ...common.types.base import Base
from ...organization_base.types.environment import Environment
from ...organization_base.types.organization_id import OrganizationId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Account(Base):
    id: AccountId
    fullname: str = pydantic.Field()
    """
    Human friendly display name for this account.
    """

    organization_id: OrganizationId = pydantic.Field()
    """
    Organization that manages this Account.
    """

    environment: Environment = pydantic.Field()
    """
    Environment this account runs in.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    User defined labels that apply to this account. These values can be used in role bindings to limit the scope of permissions.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .....v_1_3_0.resources.fileactivity.resources.classes.types.file_activity import (
    FileActivity as resources_ocsf_resources_v_1_3_0_resources_fileactivity_resources_classes_types_file_activity_FileActivity,
)

FileActivity = (
    resources_ocsf_resources_v_1_3_0_resources_fileactivity_resources_classes_types_file_activity_FileActivity
)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .encryption_details_algorithm_id import EncryptionDetailsAlgorithmId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class EncryptionDetails(pydantic.BaseModel):
    """
    Details about the encrytpion methodology utilized.
    """

    algorithm: typing.Optional[str] = pydantic.Field(default=None)
    """
    The encryption algorithm used, normalized to the caption of 'algorithm_id
    """

    algorithm_id: typing.Optional[EncryptionDetailsAlgorithmId] = pydantic.Field(default=None)
    """
    The encryption algorithm used.
    """

    key_length: typing.Optional[int] = pydantic.Field(default=None)
    """
    The length of the encryption key used.
    """

    key_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the key used for encrpytion. For example, AWS KMS Key ARN.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of the encryption used.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Organization(pydantic.BaseModel):
    """
    The Organization object describes characteristics of an organization or company and its division if any. Additionally, it also describes cloud and Software-as-a-Service (SaaS) logical hierarchies such as AWS Organizations, Google Cloud Organizations, Oracle Cloud Tenancies, and similar constructs.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the organization, Oracle Cloud Tenancy, Google Cloud Organization, or AWS Organization. For example, <code> Widget, Inc. </code> or the <code> AWS Organization name </code>.
    """

    ou_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of an organizational unit, Google Cloud Folder, or AWS Org Unit. For example, the <code> GCP Project Name </code>, or <code> Dev_Prod_OU </code>.
    """

    ou_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of an organizational unit, Google Cloud Folder, or AWS Org Unit. For example, an <code> Oracle Cloud Tenancy ID </code>, <code> AWS OU ID </code>, or <code> GCP Folder ID </code>.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the organization, Oracle Cloud Tenancy, Google Cloud Organization, or AWS Organization. For example, an <code> AWS Org ID </code> or <code> Oracle Cloud Domain ID </code>.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}
